'**** A P P L I C A T I O N   N O T E   ************************************
'*
'* PHOENIXstudios PC_DIMMER 4ch LED RS232-Dimmerpack
'* Copyright (c) 2020 by Dr.-Ing. Christian Nding
'* Last change 11.08.2020
'*
'* Chip-Type: Atmel AVR ATtiny2313
'*
'*
'* Updates: http://www.pcdimmer.de
'*
'***************************************************************************
' This program is free software; you can redistribute it and/or
' modify it under the terms of the GNU General Public License
' as published by the Free Software Foundation; either version2 of
' the License, or (at your option) any later version.
'
' This program is distributed in the hope that it will be useful,
' but WITHOUT ANY WARRANTY; without even the implied warranty of
' MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
' General Public License for more details.
'
' If you have no copy of the GNU General Public License, write to the
' Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
'***************************************************************************

'***************************************************************************

'Definitionen

$regfile = "attiny2313.dat"
$crystal = 7372800
$baud = 115200
$hwstack = 40
$swstack = 16
$framesize = 32

'Programmkonstanten
Const Startaddress = 1
Const Channelcount = 4

'###############################################################################
' Variablen fr RS232-Interrupt
Dim Rs232_bytevar1 As Byte
Dim Rs232_bytevar2 As Byte
Dim Rs232_inframe(6) As Byte
Dim TurnOffLEDCounter As Byte

' EEPROM-Variablen
'Dim _dimmercurve(255) As Eram Byte At 1

' Programmvariablen
Dim Value As Byte
Dim Channelvalue(Channelcount) As Byte
Dim Channelvalue_new(Channelcount) As Byte
Dim Channelcounter(Channelcount) As Word
Dim Channelcounterpreload(Channelcount) As Word
Dim Channelcounterstep(Channelcount) As Byte

Dim Ch As Byte

'Funktionen definieren

'****************** Startup ******************
   On Urxc Rec_isr                                          'define serial receive ISR
   Enable Urxc                                              'Serial RX complete interrupt

   Config Portb.2 = Output ' OC0A
   Config PortD.4 = Output ' LED GREEN
   Config PortD.5 = Output ' OC0B
   Config PortD.6 = Output ' LED YELLOW
   Config Portb.3 = Output ' OC1A
   Config Portb.4 = Output ' OC1B

   LedGreen Alias PortD.4
   LedYellow Alias PortD.6

   Config TIMER0 = Pwm , Compare A Pwm = Clear Up, Compare B Pwm = Clear Up, Prescale = 1
   Config Timer1 = Pwm , Pwm = 8 , Compare A Pwm = Clear Up , Compare B Pwm = Clear Up , Prescale = 1

   Rs232_bytevar1 = 0

   Enable Interrupts                                        'enable interrupts to occur

   Set LedGreen
   Set LedYellow
   TurnOffLEDCounter=255

   '****************** Programmschleife ******************
   Do
      For Ch = 1 To Channelcount
         If Channelcounter(Ch)>0 Then
            ' waittime is not over for this channel - wait some more 10ms-steps
            Channelcounter(Ch) = Channelcounter(Ch) - 1
         Else
            ' Channelcounter is zero -> waittime is over
            If Channelvalue_new(Ch) < Channelvalue(Ch) Then
               ' Decrease the current channel
               Channelvalue(Ch) = Channelvalue(Ch) - Channelcounterstep(Ch)
            Elseif Channelvalue_new(Ch) > Channelvalue(Ch) Then
               ' Increase the current channel
               Channelvalue(Ch) = Channelvalue(Ch) + Channelcounterstep(Ch)
            End If

            ' check if we have reached the final value
            If Channelvalue(Ch) <> Channelvalue_new(Ch) Then
               ' channelvalue has not reached final value -> go to next iteration
               Channelcounter(Ch) = Channelcounterpreload(Ch)
            End If
         End If
      Next Ch

      ' map the array the pwm outputs
      Value = LOOKUP(Channelvalue(1), Dimmercurve)
      COMPARE0A = Value 'RED
      Value = LOOKUP(Channelvalue(2), Dimmercurve)
      COMPARE0B = Value 'GREEN
      Value = LOOKUP(Channelvalue(3), Dimmercurve)
      COMPARE1A = Value 'BLUE
      Value = LOOKUP(Channelvalue(4), Dimmercurve)
      COMPARE1B = Value 'WHITE

      ' check if we can disable the Receive-LED
      If TurnOffLEDCounter>0 Then
         Decr TurnOffLEDCounter
      Else
         Reset LedYellow
      End If

      waitms 9 ' this value should be 10ms but depending on the code within main-loop this value can be set a bit lower to achieve real 10ms
   Loop


' RS232-Empfang
Rec_isr:
If Usr.7 = 1 Then                                           'Nur wenn neue Daten empfangen wurden, diese auch verwerten
   Incr Rs232_bytevar1
   Rs232_inframe(Rs232_bytevar1) = Udr

   ' Prfen, ob Startbit gesetzt ist
   If Rs232_inframe(1) < 128 Then
      ' Startbit fehlt -> Reset des Counters und Exit
      Rs232_bytevar1 = 0
      Goto Error
   End If

   'Nun empfangene Daten verarbeiten
   If Rs232_bytevar1 >= 6 Then

      If Rs232_inframe(1) < 128 Then Goto Error             'Fehler im Datenpaket, wenn kein Startbit vorhanden
      If Rs232_inframe(6) > 127 Then Goto Error             'Fehler im Datenpaket, wenn Kanalendwert > 127

      Rs232_bytevar1 = Rs232_inframe(1) - 126                'Kanal herausfiltern (da Bit0 als Startbit)
      Rs232_bytevar1 = Rs232_bytevar1 - Startaddress

      If Rs232_bytevar1 >= 1 And Rs232_bytevar1 <= Channelcount Then
         ' we have received a packet for the defined channels -> do something with it
         Set LedYellow
         TurnOffLEDCounter = 15 ' Let the LED 15 x 10ms = 150ms on

         ' read channel increment-value (Stp)
         Rs232_bytevar2 = Rs232_inframe(2) And 3
         Channelcounterstep(Rs232_bytevar1) = Rs232_bytevar2 + 1

         ' read channel fade-wait-steps (Ftime)
         Channelcounterpreload(Rs232_bytevar1) = Rs232_inframe(3)
         Shift Channelcounterpreload(Rs232_bytevar1), Left, 7
         Channelcounterpreload(Rs232_bytevar1) = Channelcounterpreload(Rs232_bytevar1) + Rs232_inframe(4)
         Shift Channelcounterpreload(Rs232_bytevar1), Left, 7
         Channelcounterpreload(Rs232_bytevar1) = Channelcounterpreload(Rs232_bytevar1) + Rs232_inframe(5)
         Channelcounter(Rs232_bytevar1) = Channelcounterpreload(Rs232_bytevar1)


         ' read channel-endvalue
         Rs232_bytevar2 = 127 - Rs232_inframe(6)          ' channelvalue will be submitted inverted
         Shift Rs232_bytevar2, Left, 1                    ' shift channel-endvalue as it is only 7 bit long
         Channelvalue_new(Rs232_bytevar1) = Rs232_bytevar2

         ' if no dimming should be performed, just copy the value
         If Channelcounter(Rs232_bytevar1) = 0 Then
            Channelvalue(Rs232_bytevar1) = Rs232_bytevar2
         End If
      End If

      Rs232_bytevar1 = 0
   End If
End If

Error:
Return

End                                                         'end program


' ******************************************************************************
' * EEPROM-Daten                                                               *
' ******************************************************************************

' Abspeichern von Einstellungen und Tabellen
'$eeprom
'$eepromhex
'Data 0                                                      ' erstes Byte ist nicht zu gebrauchen (siehe AppNote)

Dimmercurve:
' LINEAR
'(
Data 0,  1,  2,  3,  4,  5,  6,  7,  8,  9, 10, 11, 12, 13, 14, 15
Data 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31
Data 32, 33, 34, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 45, 46, 47
Data 48, 49, 50, 51, 52, 53, 54, 55, 56, 57, 58, 59, 60, 61, 62, 63
Data 64, 65, 66, 67, 68, 69, 70, 71, 72, 73, 74, 75, 76, 77, 78, 79
Data 80, 81, 83, 84, 85, 86, 87, 88, 89, 80, 90, 91, 92, 93, 94, 95
Data 96, 97, 98, 99,100,101,102,103,104,105,106,107,108,109,110,111
Data 112,113,114,115,116,117,118,119,120,121,122,123,124,125,126,127
Data 128,129,130,131,132,133,134,135,136,137,138,139,140,141,142,143
Data 144,145,146,147,148,149,150,151,152,153,154,155,156,157,158,159
Data 160,161,162,163,164,165,166,167,168,169,170,171,172,173,174,175
Data 176,177,178,179,180,181,182,183,184,185,186,187,188,189,190,191
Data 192,193,194,195,196,197,198,199,200,201,202,203,204,205,206,207
Data 208,209,210,211,212,213,214,215,216,217,218,219,220,221,222,223
Data 224,225,226,227,228,229,230,231,232,233,234,235,236,237,238,239
Data 240,241,242,243,244,245,246,247,248,249,250,251,252,253,254,255
')

' LOGARITHMISCH 1
'(
Data 0,0,0,1,1,1,1,1,1,2,2,2,2,2,3,3
Data 3,3,3,4,4,4,4,4,5,5,5,5,5,6,6,6
Data 6,6,7,7,7,7,7,8,8,8,8,8,9,9,9,9
Data 10,10,10,10,10,11,11,11,11,12,12,12,12,13,13,13
Data 13,14,14,14,14,15,15,15,15,16,16,16,16,17,17,17
Data 17,18,18,18,18,19,19,19,19,20,20,20,21,21,21,21
Data 22,22,22,23,23,23,23,24,24,24,25,25,25,26,26,26
Data 27,27,27,28,28,28,29,29,29,30,30,30,31,31,31,32
Data 32,32,33,33,34,34,34,35,35,35,36,36,37,37,37,38
Data 38,39,39,40,40,40,41,41,42,42,43,43,44,44,44,45
Data 45,46,46,47,47,48,48,49,49,50,51,51,52,52,53,53
Data 54,54,55,56,56,57,58,58,59,59,60,61,61,62,63,64
Data 64,65,66,67,67,68,69,70,71,71,72,73,74,75,76,77
Data 78,79,80,81,82,83,84,85,86,88,89,90,91,93,94,95
Data 97,98,100,102,103,105,107,109,111,113,115,117,119,122,125,127
Data 130,133,137,141,145,149,154,159,165,172,181,191,204,223,255,255
')

' LOGARITHMISCH 2

Data 0,0,1,1,1,2,2,2,2,3,3,3,3,4,4,4
Data 5,5,5,5,6,6,6,7,7,7,8,8,8,9,9,9
Data 9,10,10,11,11,11,12,12,12,13,13,13,14,14,14,15
Data 15,16,16,16,17,17,18,18,18,19,19,20,20,21,21,21
Data 22,22,23,23,24,24,25,25,26,26,27,27,28,28,29,29
Data 30,30,31,31,32,32,33,33,34,34,35,36,36,37,37,38
Data 39,39,40,40,41,42,42,43,44,44,45,45,46,47,48,48
Data 49,50,50,51,52,52,53,54,55,55,56,57,58,59,59,60
Data 61,62,63,63,64,65,66,67,68,68,69,70,71,72,73,74
Data 75,76,77,78,79,80,81,82,83,84,85,86,87,88,89,90
Data 91,92,93,94,95,96,98,99,100,101,102,103,105,106,107,108
Data 110,111,112,113,115,116,117,119,120,121,123,124,125,127,128,130
Data 131,133,134,136,137,139,140,142,143,145,146,148,150,151,153,155
Data 156,158,160,161,163,165,167,168,170,172,174,176,178,179,181,183
Data 185,187,189,191,193,195,197,199,201,204,206,208,210,212,214,217
Data 219,221,223,226,228,230,233,235,238,240,243,245,248,250,253,255


' TRIAC DIMMERCURVE FOR HALOGEN LAMPS
'(
Data 0, 11, 15, 17, 20, 22, 24, 27, 29, 30, 32, 33, 35, 36, 38, 39
Data 40, 42, 43, 44, 45, 46, 47, 48, 50, 51, 52, 53, 54, 55, 56, 57
Data 58, 59, 59, 60, 61, 62, 63, 64, 65, 66, 66, 67, 68, 69, 70, 70
Data 71, 72, 73, 74, 74, 75, 76, 77, 77, 78, 79, 80, 80, 81, 82, 83
Data 84, 84, 85, 86, 86, 87, 88, 88, 89, 90, 90, 91, 92, 92, 93, 94
Data 94, 95, 96, 96, 97, 98, 98, 99,100,100,101,102,102,103,104,104
Data 105,106,106,107,108,108,109,109,110,111,111,112,113,113,114,114
Data 115,116,116,117,118,118,119,119,120,121,121,122,123,123,124,124
Data 125,126,126,127,127,128,129,129,130,131,131,132,132,133,134,134
Data 135,136,136,137,137,138,139,139,140,141,141,142,142,143,144,144
Data 145,146,146,147,148,148,149,150,150,151,152,152,153,154,154,155
Data 156,156,157,158,158,159,160,160,161,162,162,163,164,164,165,166
Data 166,167,168,169,170,170,171,172,173,173,174,175,176,176,177,178
Data 179,180,180,181,182,183,184,184,185,186,187,188,189,190,191,191
Data 192,193,194,195,196,197,198,199,200,202,203,204,205,206,207,208
Data 210,211,212,214,215,217,218,220,221,223,226,228,230,233,235,255
')

' SWITCHPACK
'(
Data   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0
Data   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0
Data   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0
Data   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0
Data   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0
Data   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0
Data   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0
Data   0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0
Data 255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255
Data 255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255
Data 255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255
Data 255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255
Data 255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255
Data 255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255
Data 255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255
Data 255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255
')